package com.simingtang.smt_face_ml_kit

import android.content.Context
import android.content.Intent
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.util.Log
import androidx.annotation.NonNull
import androidx.camera.core.*
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.camera.view.PreviewView
import androidx.core.content.ContextCompat
import androidx.lifecycle.LifecycleOwner
import com.google.common.util.concurrent.ListenableFuture
import com.google.mlkit.vision.common.InputImage
import com.google.mlkit.vision.face.FaceDetection
import com.google.mlkit.vision.face.FaceDetectorOptions
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors

class SmtFaceMlKitPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    private lateinit var channel: MethodChannel

    private lateinit var mFlutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    private var cameraExecutor: ExecutorService? = null
    private var sensorManager: SensorManager? = null
    private var cameraProviderFuture: ListenableFuture<ProcessCameraProvider>? = null
    private var cameraProvider: ProcessCameraProvider? = null
    private lateinit var mBinding: ActivityPluginBinding
    private lateinit var previewView: PreviewView


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        mFlutterPluginBinding = flutterPluginBinding
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "smt_face_ml_kit")
        channel.setMethodCallHandler(this)
    }

    var isNeedView = true

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "startFaceMlKit" -> {
                isNeedView = (call.arguments as? Boolean) ?: true
                if (isNeedView) {
                    if (!::previewView.isInitialized) {
                        result.error("-1", "初始化未完成", "")
                        return
                    }
                }
                initSensor()
                startCamera()
            }
            "endFaceMlKit" -> {
                finishSmtFaceMlKitActivity()
            }
        }
        result.success(null)
    }

    private fun finishSmtFaceMlKitActivity() {
        sensorManager?.unregisterListener(listener)
        cameraExecutor?.shutdown()
        cameraProvider?.unbindAll()
        cameraProviderFuture?.cancel(true)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        finishSmtFaceMlKitActivity()
    }

    //第三步：对传感器信号进行监听
    private val listener: SensorEventListener = object : SensorEventListener {
        override fun onSensorChanged(event: SensorEvent) {
            channel.invokeMethod("lightInfo", event.values[0].toString())
        }

        override fun onAccuracyChanged(sensor: Sensor, accuracy: Int) {}
    }

    private fun initSensor() {
        //第一步：获取 SensorManager 的实例
        sensorManager =
            mBinding.activity.getSystemService(Context.SENSOR_SERVICE) as SensorManager
        //第二步：获取 Sensor 传感器类型
        val sensor: Sensor = sensorManager!!.getDefaultSensor(Sensor.TYPE_LIGHT)
        //第四步：注册 SensorEventListener
        sensorManager!!.registerListener(listener, sensor, SensorManager.SENSOR_DELAY_NORMAL)
    }

    private fun startCamera() {
        cameraExecutor = Executors.newSingleThreadExecutor()

        cameraProviderFuture = ProcessCameraProvider.getInstance(mBinding.activity)
        cameraProviderFuture!!.addListener({
            cameraProvider = cameraProviderFuture!!.get()

            val imageCapture = ImageCapture.Builder()
                .build()

            val preview = if (isNeedView) {
                Preview.Builder()
                    .build()
                    .also {
                        it.setSurfaceProvider(previewView.surfaceProvider)
                    }
            } else {
                null
            }

            val realTimeOpts = FaceDetectorOptions.Builder()
                .setContourMode(FaceDetectorOptions.CONTOUR_MODE_NONE)
                .build()
            val detector = FaceDetection.getClient(realTimeOpts)

            val imageAnalyzer = ImageAnalysis.Builder()
                .build()
                .also {
                    it.setAnalyzer(cameraExecutor!!, object : ImageAnalysis.Analyzer {
                        override fun analyze(image: ImageProxy) {
                            val mediaImage = image.image
                            if (mediaImage != null) {
                                val detectorImage =
                                    InputImage.fromMediaImage(mediaImage, image.imageInfo.rotationDegrees)
                                detector.process(detectorImage)
                                    .addOnSuccessListener { faces ->
                                        val faceData = faces.map {
                                            val rectBox = it.boundingBox
                                            return@map "{" +
                                                    "\"centerX\":${rectBox.centerX()}," +
                                                    "\"centerY\":${rectBox.centerY()}," +
                                                    "\"width\":${rectBox.width()}," +
                                                    "\"height\":${rectBox.height()}," +
                                                    "\"imageWidth\":${detectorImage.height}," +
                                                    "\"imageHeight\":${detectorImage.width}" +
                                                    "}"
                                        }.toList()
                                        channel.invokeMethod("faceInfos", faceData)
                                    }
                                    .addOnFailureListener { e ->
                                        channel.invokeMethod("error", "错误信息:人脸识别错误")
                                    }.addOnCompleteListener {
                                        image.close()
                                    }
                            }
                        }
                    })
                }

            try {
                cameraProvider!!.unbindAll()

                if (preview == null) {
                    cameraProvider!!.bindToLifecycle(
                        mBinding.activity as LifecycleOwner,
                        CameraSelector.DEFAULT_FRONT_CAMERA,
                        imageCapture,
                        imageAnalyzer
                    )
                } else {
                    cameraProvider!!.bindToLifecycle(
                        mBinding.activity as LifecycleOwner,
                        CameraSelector.DEFAULT_FRONT_CAMERA,
                        preview,
                        imageCapture,
                        imageAnalyzer
                    )
                }

            } catch (exc: Exception) {
                Log.e("flutter", exc.toString())
                channel.invokeMethod("error", "错误信息：相机绑定信息出错")
            }

        }, ContextCompat.getMainExecutor(mBinding.activity))
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {

        mBinding = binding

        previewView = PreviewView(binding.activity)

        previewView.setBackgroundColor(binding.activity.resources.getColor(android.R.color.white))

        val factory = SMTCameraViewFactory(previewView)

        mFlutterPluginBinding.platformViewRegistry.registerViewFactory(
            "smt_camera_view",
            factory
        )

    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
    }
}
